@@ -48,7 +48,6 @@ string parseCommonOptions(int argc, char** argv, shared_ptr< Config > config, ve
 		("important,i", "")
 		("option,o", bpo::value< vector< string > >(&directOptions))
 		("recurse", "")
-		("purge", "")
 		("all-versions,a", "")
 		("no-all-versions", "")
 		("target-release", bpo::value< string >(&targetRelease))
@@ -69,14 +68,20 @@ string parseCommonOptions(int argc, char** argv, shared_ptr< Config > config, ve
 				.style(bpo::command_line_style::default_style & ~bpo::command_line_style::allow_guessing)
 				.positional(positionalOptions).allow_unregistered().run();
 		bpo::store(parsed, variablesMap);
-		unparsed = bpo::collect_unrecognized(parsed.options, bpo::exclude_positional);
-		if (variablesMap.count("arguments"))
-		{
-			vector<std::string> arguments = variablesMap["arguments"].as< vector< string > >();
-			unparsed.insert(unparsed.begin(), arguments.begin(), arguments.end());
-		}
 		bpo::notify(variablesMap);
 
+		{ // do not pass 'command' further
+			FORIT(optionIt, parsed.options)
+			{
+				if (optionIt->string_key == "command")
+				{
+					parsed.options.erase(optionIt);
+					break;
+				}
+			}
+		}
+		unparsed = bpo::collect_unrecognized(parsed.options, bpo::include_positional);
+
 		{ // processing
 			if (command.empty())
 			{
@@ -94,10 +99,6 @@ string parseCommonOptions(int argc, char** argv, shared_ptr< Config > config, ve
 			{
 				config->setScalar("apt::default-release", targetRelease);
 			}
-			if (variablesMap.count("purge"))
-			{
-				config->setScalar("cupt::worker::purge", "yes");
-			}
 			if (variablesMap.count("all-versions"))
 			{
 				config->setScalar("apt::cache::allversions", "yes");
@@ -154,7 +155,7 @@ string parseCommonOptions(int argc, char** argv, shared_ptr< Config > config, ve
 }
 
 bpo::variables_map parseOptions(const Context& context, bpo::options_description options,
-		vector< string >& arguments)
+		vector< string >& arguments, std::function< pair< string, string > (const string&) > extraParser)
 {
 	bpo::options_description argumentOptions("");
 	argumentOptions.add_options()
@@ -172,7 +173,7 @@ bpo::variables_map parseOptions(const Context& context, bpo::options_description
 	{
 		bpo::parsed_options parsed = bpo::command_line_parser(context.unparsed)
 				.style(bpo::command_line_style::default_style & ~bpo::command_line_style::allow_guessing)
-				.options(all).positional(positionalOptions).run();
+				.options(all).positional(positionalOptions).extra_parser(extraParser).run();
 		bpo::store(parsed, variablesMap);
 	}
 	catch (const bpo::unknown_option& e)
